<?php
/**
 * Options Class File
 *
 * This file will contain Options Class.
 *
 * @package    Customizely
 * @author     KitThemes (https://www.kitthemes.com/)
 * @copyright  Copyright (c) 2019, KitThemes
 * @since      1.0.0
 */

namespace Customizely;

/**
 * Options Class
 *
 * This will contain all option generated by user.
 * All options will be set, get and remove from here.
 *
 * @since 1.0.0
 */
class Options {
	/**
	 * Customizer Panels
	 *
	 * It will hold all user registed customizer
	 * panels with ID and settings.
	 *
	 * @since 1.0.0
	 * @access private
	 *
	 * @var array
	 */
	private $panels = [];

	/**
	 * Last Panel ID
	 *
	 * If section don't set panel,
	 * it will add into section as panel id.
	 *
	 * @since 1.0.0
	 * @access private
	 *
	 * @var string
	 */
	private $panel_id = '';

	/**
	 * Customizer Sections
	 *
	 * It will hold all user registed customizer
	 * sections with ID and settings.
	 *
	 * @since 1.0.0
	 * @access private
	 *
	 * @var array
	 */
	private $sections = [];

	/**
	 * Last Section ID
	 *
	 * If control don't set section,
	 * it will add into control as section id.
	 *
	 * @since 1.0.0
	 * @access private
	 *
	 * @var string
	 */
	private $section_id = '';

	/**
	 * Customizer Controls
	 *
	 * It will hold all user registed customizer
	 * controls with ID, arguments and settings.
	 *
	 * @since 1.0.0
	 * @access private
	 *
	 * @var array
	 */
	private $controls = [];

	/**
	 * Constructor Method for Customize
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return void
	 */
	public function __construct() {}

	/**
	 * Add panel
	 *
	 * This method will add panel to panels
	 * array for later use.
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param string $id Panel unique ID.
	 * @param array  $settings Panel settings.
	 *
	 * @return Customizely\Options
	 */
	public function add_panel( $id, $settings = [] ) {
		if ( isset( $this->panels[ $id ] ) ) {
			return $this;
		}

		$this->panels[ $id ] = $settings;
		$this->panel_id      = $id;

		return $this;
	}

	/**
	 * End Panel.
	 *
	 * End panel to start new panel or section.
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return Customizely\Options
	 */
	public function end_panel() {
		$this->panel_id = '';

		return $this;
	}

	/**
	 * Add section
	 *
	 * This method will add section to sections
	 * array for later use.
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param string $id Section unique ID.
	 * @param array  $settings Section settings.
	 *
	 * @return Customizely\Options
	 */
	public function add_section( $id, $settings = [] ) {
		if ( isset( $this->sections[ $id ] ) ) {
			return $this;
		}

		if ( ! isset( $settings['panel'] ) && ! empty( $this->panel_id ) ) {
			$settings['panel'] = $this->panel_id;
		}

		$this->sections[ $id ] = $settings;
		$this->section_id      = $id;

		return $this;
	}

	/**
	 * End Section.
	 *
	 * End section to start new section.
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return Customizely\Options
	 */
	public function end_section() {
		$this->section_id = '';

		return $this;
	}

	/**
	 * Add Control
	 *
	 * This method will add control to controls
	 * array for later use.
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param string $id Panel unique ID.
	 * @param array  $args Panel settings.
	 * @param string $settings Panel settings.
	 *
	 * @return Customizely\Options
	 */
	public function add_control( $id, $args = [], $settings = [] ) {
		if ( isset( $this->sections[ $id ] ) ) {
			return $this;
		}

		if ( ! isset( $args['section'] ) && ! empty( $this->section_id ) ) {
			$args['section'] = $this->section_id;
		}

		$args['settings'] = $id;

		$this->controls[ $id ] = [
			'args'     => $args,
			'settings' => $settings,
		];

		return $this;
	}

	/**
	 * Get Panels
	 *
	 * Return all panels to register with this class
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return  array  Panels array
	 */
	public function get_panels() {
		return $this->panels;
	}

	/**
	 * Get Sections
	 *
	 * Return all sections to register with this class
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return  array  Sections array
	 */
	public function get_sections() {
		return $this->sections;
	}

	/**
	 * Get Controls
	 *
	 * Return all controls to register with this class
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return  array  Controls array
	 */
	public function get_controls() {
		return $this->controls;
	}
}
